/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 */

function getParameterByName(name)
{
  name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
  var regexS = "[\\?&]" + name + "=([^&#]*)";
  var regex = new RegExp(regexS);
  var results = regex.exec(window.location.search);
  if(results == null)
    return "";
  else
    return decodeURIComponent(results[1].replace(/\+/g, " "));
}



Ext.define('DSS.addon.SceneResponder', {

    extend: 'DSS.addon.Framework',

    config: {
        appName: 'SceneResponder',
        appId: 'system-addon-scene-responder',
        appVersion: '1.4.6',
        appIcon: 'images/app_icon.png'
    },

    getHelp: function() {
        return Ext.create('Ext.Component', {
            whiteSpace : 'normal',
            padding : '0 20 0 0', //top right bottom left
            loader: {
                url: 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad: true,
                disableCaching: false
            }
        });
    },

    getContent: function() {
        var me=this;
        /* Parameter:
         *   * editor config
         *   * button config
         *   * store config   
         *   * columns
         *   * defaultSortCol
         * */
       
        var oConfig={
                defaultSortCol:'name',
                disabledProperty:'disabled',
                inactiveField:'inactive',
                storeConfig: {
                    storeId : 'sceneresponderStore',
                    fields: [
                        { name: 'id' },
                        { name: 'disabled',  sortType: 'asBoolean' },
                        { name: 'name',     sortType: 'asUCText'  },
                        { name: 'actions',  sortType: 'asAction'  },
                        { name: 'triggers', sortType: 'asTrigger' },
                        { name: 'delay',    sortType: 'asTimespan' },
                        { name: 'conditions',  sortType: 'asCondition' },
                        { name: 'singularTriggered',  sortType: 'asBoolean' },
                        { name: 'initialTriggered',  sortType: 'asBoolean' },
                        { name: 'doubleChecked',  sortType: 'asBoolean' },
                        { name: 'inactive',  sortType: 'asBoolean' },
                        {name: 'grouping',  sortType: 'asUCText' },
                        { name: 'scope', sortType: 'asUCText'}
                    ]
                },
                multiselect:true,                
                columns: [{
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                },  {
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn',
                    width : 200
                }, {
                    header: _('Trigger'),
                    dataIndex: 'triggers',
                    xtype: 'triggercolumn',
                    maintainFlex: true,
                    flex: 3
                },  {
                    header : _('Activity'),
                    dataIndex: 'actions', 
                    xtype: 'actioncolumn',
                    maintainFlex : true,
                    flex : 3
                },{
                    header: _('Conditions'),
                    dataIndex: 'conditions', 
                    xtype: 'conditioncolumn',
                    maintainFlex: true,
                    flex: 3,
                    customRendererOptions:{hideEnabled:true,hideWeekdays:false,hideGenericSystemStates:false,hideZoneStates:false,hideTimeframe:false,filterSystemState:[],
                    	stateTitles:[{name:'daylight',value:true,title:_('grid_daylight_true')},{name:'daylight',value:false,title:_('grid_daylight_false')},
                    	             {name:'daynight',value:true,title:_('grid_daynight_true')},{name:'daynight',value:false,title:_('grid_daynight_false')},
                    	             {name:'panic',value:'1',title:_('grid_panic_active')},{name:'panic',value:'2',title:_('grid_panic_inactive')},
                    	             {name:'presence',value:'present',title:_('grid_presence_present')},{name:'presence',value:'absent',title:_('grid_presence_absent')},
                    	             {name:'fire',value:'1',title:_('grid_fire_active')},{name:'fire',value:'2',title:_('grid_fire_inactive')},
                    	             {name:'hail',value:'1',title:_('grid_hail_active')},{name:'hail',value:'2',title:_('grid_hail_inactive')},
                    	             {name:'wind',value:'1',title:_('grid_wind_active')},{name:'wind',value:'2',title:_('grid_wind_inactive')},
                    	             {name:'rain',value:'1',title:_('grid_rain_active')},{name:'rain',value:'2',title:_('grid_rain_inactive')},
                    	             {name:'frost',value:'1',title:_('grid_frost_active')},{name:'frost',value:'2',title:_('grid_frost_inactive')},
                    	             {name:'sun',value:'1',title:_('grid_sun_active')},{name:'sun',value:'2',title:_('grid_sun_inactive')},
                    	             {name:'service',value:'1',title:_('grid_service_active')},{name:'service',value:'2',title:_('grid_service_inactive')},
                    	             {name:'alarm',value:'1',title:_('grid_alarm_active')},{name:'alarm',value:'2',title:_('grid_alarm_inactive')},
                    	             {name:'alarm2',value:'1',title:_('grid_alarm2_active')},{name:'alarm2',value:'2',title:_('grid_alarm2_inactive')},
                    	             {name:'alarm3',value:'1',title:_('grid_alarm3_active')},{name:'alarm3',value:'2',title:_('grid_alarm3_inactive')},
                    	             {name:'alarm4',value:'1',title:_('grid_alarm4_active')},{name:'alarm4',value:'2',title:_('grid_alarm4_inactive')},
                    	             {name:'holiday', value:'on',title:_('grid_holiday_on')},{name:'holiday', value:'off',title:_('grid_holiday_off')}]}
                },{
                    header: _('Delay'),
                    dataIndex: 'delay',
                    xtype: 'timespancolumn',
                    width: 80
                }],     
                buttonText: {
                    newButton: _('New automaton'),
                    editButton: _('Edit automaton'),
                    enableButton:_('Enable automaton'),
                    disableButton:_('Disable automaton'),
                    testButton: _('Test automaton'),
                    deleteButton: _('Delete automaton')
                },
                editorConfig:{
                    title : _('Edit automaton'),
                    staticControls : [ {
                        type : 'text',
                        label : _('Name'),
                        id : 'name'
                    }, {
                        type: 'hidden',
                        label: 'id',
                        id: 'id'
                    }],
                    tabControls : [{
                        tabLabel : _('Trigger'),
                        tabDescription : _('Choose the triggering activity for the message. You can choose more than one activity as trigger.'),
                        content : [{
                            type : 'TriggerConfigurator',
                            id : 'triggers',
                            flex : true
                       }, {
                            type: 'timespan',
                            label: _('Delay'),
                            id: 'delay'
                        }]
                    }, {
                        tabLabel : _('Activity'),
                        tabDescription : _('Choose the activities, which should be executed.'),
                        content : [{
                            type : 'ActionConfigurator',
                            id : 'actions',
                            flex : true,
                        	heatingOperationMode:true,
                            withDelay : true
                        }]
                    }, {
                        tabLabel : _('Conditions'),
                        tabDescription : _('Choose the conditions.'),
                        content : [{
                            type : 'ConditionConfigurator',
                            id : 'conditions',
                            conditions:[
                                        {type:'singleItem',title:_('state_daynight'),entry:[
                                              {type:'state',name:'daylight',value:true,title:Ext.String.format2(_('daynight_bright'),{
		                                            	duskTime:dss.staticDataModel.getDusk(),
		                                            	dawnTime:dss.staticDataModel.getDawn()
		                                            }),gridText:Ext.String.format2(_('grid_daynight_twilight_true'),{
		                                            	duskTime:dss.staticDataModel.getDusk(),
		                                            	dawnTime:dss.staticDataModel.getDawn()
		                                            })},
                                              {type:'state',name:'daylight',value:false,title:Ext.String.format2(_('daynight_dark'),{
	                                            	duskTime:dss.staticDataModel.getDusk(),
	                                            	dawnTime:dss.staticDataModel.getDawn()
	                                            }),gridText:Ext.String.format2(_('grid_daynight_twilight_false'),{
	                                            	duskTime:dss.staticDataModel.getDusk(),
	                                            	dawnTime:dss.staticDataModel.getDawn()
	                                            })},
                                              {type:'state',name:'daynight',value:true,title:Ext.String.format2(_('daynight_day'),{
	                                              	sunriseTime:dss.staticDataModel.getSunrise(),
	                                            	sunsetTime:dss.staticDataModel.getSunset()
	                                            }),gridText:Ext.String.format2(_('grid_daynight_daynight_true'),{
	                                              	sunriseTime:dss.staticDataModel.getSunrise(),
	                                            	sunsetTime:dss.staticDataModel.getSunset()
	                                            })},
                                              {type:'state',name:'daynight',value:false,title:Ext.String.format2(_('daynight_night'),{
	                                              	sunriseTime:dss.staticDataModel.getSunrise(),
	                                            	sunsetTime:dss.staticDataModel.getSunset()
	                                            }),gridText:Ext.String.format2(_('grid_daynight_daynight_false'),{
	                                              	sunriseTime:dss.staticDataModel.getSunrise(),
	                                            	sunsetTime:dss.staticDataModel.getSunset()
	                                            })}
	                                            
	                                            ]},
                                        {type:'timeframe',title:_('Timeframe')},
                                        {type:'weekdays',title:_('Weekdays')},
                                        {type:'deviceState',title:_('automatic devices')},                                                                                
                                        {type:'singleItem',title:_('state_presence'),entry: [
                                               {type:'state',name:'presence',value:'present',title:_('presence_present'),gridText:_('grid_presence_present')},
                                               {type:'state',name:'presence',value:'absent',title:_('presence_absent'),gridText:_('grid_presence_absent')}]},                                               
                                        {type:'singleItem',title:_('state_holiday'),entry: [
                                               {type:'state',name:'holiday',value:'on',title:_('holiday_on'),gridText:_('grid_holiday_on')},
                                               {type:'state',name:'holiday',value:'off',title:_('holiday_off'),gridText:_('grid_holiday_off')}]},
	                                   {type:'multiItem',title:_('weather trigger'),entry: [
                                               {type:'state',name:'rain',value:'1',title:_('rain active'),gridText:_('grid_rain_active'),group:1},
                                               {type:'state',name:'rain',value:'2',title:_('rain inactive'),gridText:_('grid_rain_inactive'),group:1},
                                               {type:'state',name:'wind',value:'1',title:_('wind active'),gridText:_('grid_wind_active'),group:2},
                                               {type:'state',name:'wind',value:'2',title:_('wind inactive'),gridText:_('grid_wind_inactive'),group:2},
                                               {type:'state',name:'hail',value:'1',title:_('hail active'),gridText:_('grid_hail_active'),group:3},
                                               {type:'state',name:'hail',value:'2',title:_('hail inactive'),gridText:_('grid_hail_inactive'),group:3},
                                               {type:'state',name:'frost',value:'1',title:_('frost active'),gridText:_('grid_frost_active'),group:4},
                                               {type:'state',name:'frost',value:'2',title:_('frost inactive'),gridText:_('grid_frost_inactive'),group:4},
                                               {type:'state',name:'sun',value:'1',title:_('sun active'),gridText:_('grid_sun_active'),group:5},
                                               {type:'state',name:'sun',value:'2',title:_('sun inactive'),gridText:_('grid_sun_inactive'),group:5}]},
                                               {type:'state',name:'service',value:'1',title:_('service active'),gridText:_('grid_service_active'),group:6},
                                               {type:'state',name:'service',value:'2',title:_('service inactive'),gridText:_('grid_service_inactive'),group:6},
	                                   {type:'multiItem',title:_('Security'),entry: [
                                              {type:'state',name:'fire',value:'1',title:_('fire active'),gridText:_('grid_fire_active'),group:1},
                                              {type:'state',name:'fire',value:'2',title:_('fire inactive'),gridText:_('grid_fire_inactive'),group:1},
                                               {type:'state',name:'panic',value:'1',title:_('panic_active'),gridText:_('grid_panic_active'),group:2},
                                               {type:'state',name:'panic',value:'2',title:_('panic_inactive'),gridText:_('grid_panic_inactive'),group:2},
                                               {type:'state',name:'alarm',value:'1',title:_('alarm_active'),gridText:_('grid_alarm_active'),group:3},
                                               {type:'state',name:'alarm',value:'2',title:_('alarm_inactive'),gridText:_('grid_alarm_inactive'),group:3},
                                               {type:'state',name:'alarm2',value:'1',title:_('alarm2_active'),gridText:_('grid_alarm2_active'),group:4},
                                               {type:'state',name:'alarm2',value:'2',title:_('alarm2_inactive'),gridText:_('grid_alarm2_inactive'),group:4},
                                               {type:'state',name:'alarm3',value:'1',title:_('alarm3_active'),gridText:_('grid_alarm3_active'),group:5},
                                               {type:'state',name:'alarm3',value:'2',title:_('alarm3_inactive'),gridText:_('grid_alarm3_inactive'),group:5},
                                               {type:'state',name:'alarm4',value:'1',title:_('alarm4_active'),gridText:_('grid_alarm4_active'),group:6},
                                               {type:'state',name:'alarm4',value:'2',title:_('alarm4_inactive'),gridText:_('grid_alarm4_inactive'),group:6}]},
                                        {type:'zonestate',title:_('ZoneState')},
                                        {type:'clusterstate',title:_('Cluster State')}
                            ],
                            flex : true
                        }]
                    }]
                
            	/* conditions: [
            	 * 	 {type: <[enabler,weekdays,timeframe,zonestate]>, <[defaultValue:true]>, <[readonly:true]>}
            	 *   {type: <[singleItem,multiItem]>, entry: [{type:state,name:'<statename>',value:<['active','inactive']>, defaultValue:[{name,state}]}]
            	 *   |type: deviceState, <filterFunc:callback>, defaultValue:[{name,state}]
            	 * ]
            	 * */
                
                },
                listeners: {
                    'loadDataStore':function (grid){
                        me.loadParameter(grid);
                    },
                    'saveEntry':function (grid,values){                        
                        me.waitAndReload(grid,{
                            actions : 'save',
                            value : Ext.JSON.encode(values)
                        });
                    },
                    'enableEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'enable',
                            id : Ext.JSON.encode(idVal)
                        });
                     },
                    'disableEntry':function (grid,entry){
                     	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'disable',
                            id : Ext.JSON.encode(idVal)
                        });
                    },
                    'deleteEntry':function (grid,entry){
                     	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('id');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('id'));
                    	}
                        me.waitAndReload(grid,{
                            actions : 'delete',
                            id : Ext.JSON.encode(idVal)
                        });
                    },
                    'testEntry':function (grid,entry){
                        dss.ajaxAssyncRequestWithoutAnswer("/json/event/raise",
                                {name:'action_execute', parameter:  "path=/scripts/system-addon-scene-responder/entries/" + entry.get('id') + ";ignoreConditions=ignoreConditions"});                       
                    }
                }
        };
        
        if (getParameterByName('mode')=='expert') {
            oConfig.editorConfig.tabControls[0].content.splice( 1,0,{
                type: 'checkbox',
                title: _('multiple triggered'),
                label: _('should be executed only once when triggered multiple times within delay, but delay starts with the last trigger'),
                id: 'singularTriggered'
            },{
                type: 'checkbox',
                title: _('only first'),
                label: _('should be executed only once when triggered multiple times within delay, but delay starts with the first trigger'),
                id: 'initialTriggered'
            });
            oConfig.editorConfig.tabControls[0].content[0].enableUsergroup=true;
            oConfig.editorConfig.tabControls[0].content[0].ignorePresence=true;
            oConfig.editorConfig.tabControls[0].content[0].userStates=true;
            oConfig.editorConfig.tabControls[1].content[0].withDelay=true;
            oConfig.editorConfig.tabControls[1].content[0].enableUsergroup=true;
            oConfig.editorConfig.tabControls[1].content[0].ignorePresence=true;
            oConfig.editorConfig.tabControls[1].content[0].userStates=true;
            oConfig.editorConfig.tabControls[1].content[0].heatingExtendedOperationMode=true;
            oConfig.editorConfig.tabControls[1].content[0].UMRExtendetFunctions=true;
            oConfig.editorConfig.tabControls[2].content[0].ignorePresence=true;
            oConfig.editorConfig.tabControls[2].content[0].conditions.push(
            		{type:'userDefinedState'});
            oConfig.editorConfig.tabControls[2].content[0].fFeatureEnabled={
                    enabler:true,
                    weekdays:true,
                    timeframe:true,
                    systemstate:true,
                    zonestate:true,
                    userDefinedStates:true};
            /*
            oConfig.editorConfig.tabControls[2].content[0].conditions.push(
            		{type:'plainStates',title:_('new State Mechanism'),
            			states:[{statetype:'system',name:'holiday'},
            			        {statetype:'app',name:['*'],appid:'user-defined-states'}]});*/
            oConfig.columns.push( {
                header : _('Source App'),
                dataIndex : 'scope',
                xtype: 'textcolumn',
                width : 100
            } );
            oConfig.grouperField='grouping';
        } else {        
        	oConfig.scope={fieldID:'scope',value:'system-addon-scene-responder'};
        }
        
        return Ext.create('DSS.gridPanel', oConfig);
    },
    waitAndReload : function(grid,parameter) {
    	grid.setStatus(_('Saving ...'));
    	grid.setDisabled(true);
    	
    	var me = this;
    	dss.pushAndQuery('system-addon-scene-responder.config',parameter,
    			         'system-addon-scene-responder.saved',function(eventData) {
    		 me.loadParameter(grid);
    		 return true;
    	});    	
    },
    loadParameter : function(grid) {
    	grid.setStatus(_('Loading ...'));
    	grid.setDisabled(true);
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/system-addon-scene-responder/entries/*(*)/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        var oResultObject = dss.parsePropertyQueryAnswer(result, [
            {name: 'name', type: 'simple'},
            {name: 'id', type: 'simple'},
            {name: 'singularTriggered', type: 'simple'},
            {name: 'initialTriggered', type: 'simple'},
            {name: 'doubleChecked', type: 'simple'},
            {name: 'disabled', type: 'condition/enabled'},
            {name: 'delay', type: 'simple'},
            {name: 'actions', type: 'action'},
            {name: 'conditions', type: 'condition'},
            {name: 'triggers', type: 'trigger'},
            {name: 'scope', type:'simple'}
        ]);
        for (var iIndex=0;iIndex<oResultObject.length;iIndex++) {
            oResultObject[iIndex].grouping=unescape(oResultObject[iIndex].name).split(' ')[0];
            oResultObject[iIndex].inactive=false;
        }
        grid.loadParameter(oResultObject);
        grid.setStatus(_('Ready'));
        grid.setDisabled(false);
    }
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/scene-responder.po']);
    var sceneresponder = Ext.create('DSS.addon.SceneResponder');
    sceneresponder.initPage();
});

